#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
IG Markets REST API sample with Python
2015 FemtoTrader
"""

from trading_ig import IGService
from trading_ig.config import config
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# if you need to cache to DB your requests
from datetime import timedelta
import requests_cache


def main():
    logging.basicConfig(level=logging.DEBUG)

    expire_after = timedelta(hours=1)
    session = requests_cache.CachedSession(
        cache_name="cache", backend="sqlite", expire_after=expire_after
    )
    # set expire_after=None if you don't want cache expiration
    # set expire_after=0 if you don't want to cache queries

    # config = IGServiceConfig()

    # no cache
    ig_service = IGService(
        config.username, config.password, config.api_key, config.acc_type, acc_number=config.acc_number
    )

    # if you want to globally cache queries
    # ig_service = IGService(config.username, config.password, config.api_key, config.acc_type, session)

    ig_service.create_session()
    # ig_stream_service.create_session(version='3')

    accounts = ig_service.fetch_accounts()
    print("accounts:\n%s" % accounts)

    # account_info = ig_service.switch_account(config.acc_number, False)
    # print(account_info)

    # open_positions = ig_service.fetch_open_positions()
    # print("open_positions:\n%s" % open_positions)

    print("")

    # working_orders = ig_service.fetch_working_orders()
    # print("working_orders:\n%s" % working_orders)

    print("")

    # epic = 'CS.D.EURUSD.MINI.IP'
    epic = "IX.D.ASX.IFM.IP"  # US (SPY) - mini
    #epic = "CS.D.GBPUSD.CFD.IP"  # sample CFD epic

    resolution = "D"
    # see from pandas.tseries.frequencies import to_offset
    # resolution = 'H'
    # resolution = '1Min'

    num_points = 10
    response = ig_service.fetch_historical_prices_by_epic_and_num_points(
        epic, resolution, num_points
    )
    # Exception: error.public-api.exceeded-account-historical-data-allowance

    # if you want to cache this query
    # response = ig_service.fetch_historical_prices_by_epic_and_num_points(epic, resolution, num_points, session)

    # df_ask = response['prices']['ask']
    # print("ask prices:\n%s" % df_ask)

    # (start_date, end_date) = ('2015-09-15', '2015-09-28')
    # response = ig_service.fetch_historical_prices_by_epic_and_date_range(epic, resolution, start_date, end_date)

    # if you want to cache this query
    # response = ig_service.fetch_historical_prices_by_epic_and_date_range(epic, resolution, start_date, end_date, session)
    # df_ask = response['prices']['ask']
    # print("ask prices:\n%s" % df_ask)


if __name__ == "__main__":
    main()



from trading_ig.rest import IGService, ApiExceededException
from trading_ig.config import config
from tenacity import Retrying, wait_exponential, retry_if_exception_type

DEFAULT_RETRY = Retrying(wait=wait_exponential(),
        retry=retry_if_exception_type(ApiExceededException))


def display_top_level_nodes():
    ig_service = get_session()

    response = ig_service.fetch_top_level_navigation_nodes()
    df = response['nodes']
    for record in df.to_dict('records'):
        print(f"{record['name']} [{record['id']}]")


def display_all_epics():
    ig_service = get_session()
    response = ig_service.fetch_top_level_navigation_nodes()
    df = response['nodes']
    for record in df.to_dict('records'):
        print(f"{record['name']} [{record['id']}]")
        display_epics_for_node(record['id'], space='  ', ig_service=ig_service)


def display_epics_for_node(node_id=0, space='', ig_service=None):
    if ig_service is None:
        ig_service = get_session()

    sub_nodes = ig_service.fetch_sub_nodes_by_node(node_id)

    if sub_nodes['nodes'].shape[0] != 0:
        rows = sub_nodes['nodes'].to_dict('records')
        for record in rows:
            print(f"{space}{record['name']} [{record['id']}]")
            display_epics_for_node(record['id'], space=space + '  ', ig_service=ig_service)

    if sub_nodes['markets'].shape[0] != 0:
        cols = sub_nodes['markets'].to_dict('records')
        for record in cols:
            print(f"{space}{record['instrumentName']} ({record['expiry']}): {record['epic']}")


def get_session():
    ig_service = IGService(
        config.username,
        config.password,
        config.api_key,
        config.acc_type,
        acc_number=config.acc_number,
        retryer=DEFAULT_RETRY)
    ig_service.create_session(version='3')
    return ig_service

if __name__ == "__main__":

    display_top_level_nodes()
    #display_all_epics()

    """
    Weekend Markets [191926749]
    Indices [97601]
    Forex [195235]
    Commodities Metals Energies [101515]
    Cryptocurrency [668394]
    Bonds and Moneymarket [108092]
    ETFs, ETCs & Trackers [184730]
    Shares - UK [180500]
    Shares - UK International (IOB) [97695]
    Shares - US (All Sessions) [298158]
    Shares - US [97477]
    Shares - Austria [114058]
    Shares - Belgium [114036]
    Shares - Canada [103185]
    Shares - Denmark [419769]
    Shares - Finland [99514]
    Shares - France [113659]
    Shares - LSE (UK) [172904]
    Shares - Germany [97466]
    Shares - Greece [100437]
    Shares - Hong Kong [105775]
    Shares - Ireland (LSE) [421257]
    Shares - Ireland (Euronext Dublin) [99578822]
    Shares - Netherlands [105509]
    Shares - New Zealand [127489792]
    Shares - Norway [99808]
    Shares - Portugal [99787]
    Shares - Singapore [105781]
    Shares - South Africa [100066]
    Shares - Sweden [113681]
    Shares - Switzerland [99814]
    IPOs [324080]
    Options (Australia 200) [77976799]
    Options (Eu Stocks 50) [245938]
    Options (France 40) [188760]
    Options (FTSE) [122250]
    Options (Germany) [97612]
    Options (HS 50) [92462573]
    Options (Japan 225) [111915265]
    Options (Netherlands 25) [236490]
    Options (Sweden 30) [319963]
    Options (Taiwan Index) [157168018]
    Options (US 500) [267039]
    Options (US Tech 100) [89291253]
    Options (Wall St) [122505]
    Options on FX Majors [255072]
    Options (Volatility Index) [56719751]
    Options on Metals, Energies [195913]
    """

    #display_epics_for_node(195913)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
IG Markets Stream API sample with Python
2015 FemtoTrader
"""

import logging

from trading_ig import IGService, IGStreamService
from trading_ig.config import config
from trading_ig.lightstreamer import Subscription


# A simple function acting as a Subscription listener
def on_prices_update(item_update):
    # print("price: %s " % item_update)
    print(
        "{stock_name:<19}: Time {UPDATE_TIME:<8} - "
        "Bid {BID:>5} - Ask {OFFER:>5}".format(
            stock_name=item_update["name"], **item_update["values"]
        )
    )


def on_account_update(balance_update):
    print("balance: %s " % balance_update)


def main():
    logging.basicConfig(level=logging.INFO)
    # logging.basicConfig(level=logging.DEBUG)

    ig_service = IGService(
        config.username, config.password, config.api_key, config.acc_type, acc_number=config.acc_number
    )

    ig_stream_service = IGStreamService(ig_service)
    ig_stream_service.create_session()
    #ig_stream_service.create_session(version='3')

    # Making a new Subscription in MERGE mode
    subscription_prices = Subscription(
        mode="MERGE",
        #items=["L1:CS.D.GBPUSD.CFD.IP", "L1:CS.D.USDJPY.CFD.IP"], # sample CFD epics
        items=["L1:CS.D.GBPUSD.TODAY.IP", "L1:IX.D.FTSE.DAILY.IP"], # sample spreadbet epics
        fields=["UPDATE_TIME", "BID", "OFFER", "CHANGE", "MARKET_STATE"],
    )

    # Adding the "on_price_update" function to Subscription
    subscription_prices.addlistener(on_prices_update)

    # Registering the Subscription
    sub_key_prices = ig_stream_service.ls_client.subscribe(subscription_prices)

    # Making an other Subscription in MERGE mode
    subscription_account = Subscription(
        mode="MERGE", items=["ACCOUNT:" + config.acc_number], fields=["AVAILABLE_CASH"],
    )

    # Adding the "on_balance_update" function to Subscription
    subscription_account.addlistener(on_account_update)

    # Registering the Subscription
    sub_key_account = ig_stream_service.ls_client.subscribe(subscription_account)

    input(
        "{0:-^80}\n".format(
            "HIT CR TO UNSUBSCRIBE AND DISCONNECT FROM \
    LIGHTSTREAMER"
        )
    )

    # Disconnecting
    ig_stream_service.disconnect()


if __name__ == "__main__":
    main()
